package at.hl.okb.wsclient;

import static at.hl.okb.wsclient.Utilities.createNameFilter;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import at.hl.wsclient.okb.Algorithm;
import at.hl.wsclient.okb.AlgorithmClass;
import at.hl.wsclient.okb.AlgorithmParameter;
import at.hl.wsclient.okb.ArrayOfAlgorithmParameter;
import at.hl.wsclient.okb.IOKBService;
import at.hl.wsclient.okb.NamedOKBItem;
import at.hl.wsclient.okb.OKBService;
import at.hl.wsclient.okb.ObjectFactory;
import at.hl.wsclient.okb.Platform;
import at.hl.wsclient.okb.Problem;
import at.hl.wsclient.okb.ProblemClass;

import com.google.common.collect.Iterables;

public class OKBServiceClient {
	OKBService srv = null;
	IOKBService port = null;
	ObjectFactory factory = new ObjectFactory();

	public OKBServiceClient() {
		this.srv = new OKBService();
		this.port = srv.getWSHttpBindingIOKBService1();
	}

	public Long getPlatformId(String platformName) {
		Platform returnValue = Iterables.find(
				port.getPlatforms().getPlatform(),
				createNameFilter(platformName), null);

		return returnValue != null ? returnValue.getId() : null;
	}

	public Long getProblemId(String problemName) {
		Problem returnValue = Iterables.find(port.getProblems().getProblem(),
				createNameFilter(problemName), null);

		return returnValue != null ? returnValue.getId() : null;
	}

	public Long getProblemClassId(String problemClass) {
		ProblemClass returnValue = Iterables.find(port.getProblemClasses()
				.getProblemClass(), createNameFilter(problemClass), null);

		return returnValue != null ? returnValue.getId() : null;

	}

	public Long getAlgorithmClassId(String algorithmClassName) {
		AlgorithmClass returnValue = Iterables.find(port.getAlgorithmClasses()
				.getAlgorithmClass(), createNameFilter(algorithmClassName),
				null);

		return returnValue != null ? returnValue.getId() : null;

	}

	public Long getProblem(String problemName, Long problemClassId,
			Long platformId) {
		Problem problem = null;
		Iterator<Problem> it = port.getProblems().getProblem().iterator();

		while (it.hasNext() && problem == null) {
			Problem tmpProblem = it.next();
			if (platformId.equals(tmpProblem.getPlatformId())
					&& problemName.equals(tmpProblem.getName().getValue())
					&& problemClassId.equals(tmpProblem.getProblemClassId())) {

				problem = tmpProblem;
			}
		}
		return problem != null ? problem.getId() : null;
	}

	public Long getAlgorithmId(String algorithmName, /* Long algorithmClassId, */
	Long platformId) {
		Algorithm algorithm = null;

		Iterator<Algorithm> it = port.getAlgorithms().getAlgorithm().iterator();

		while (it.hasNext() && algorithm == null) {
			Algorithm tmpAlgo = it.next();
			if (platformId.equals(tmpAlgo.getPlatformId())
					&& algorithmName.equals(tmpAlgo.getName().getValue())
			// && algorithmClassId.equals(tmpAlgo.getAlgorithmClassId())
			) {
				algorithm = tmpAlgo;
			}
		}

		return algorithm != null ? algorithm.getId() : null;

	}

	// public Map<String, ECJDataTypeEnum> getAlgorithParameter(Long
	// algorithmId) {
	// Map<String, AlgorithmParameter> returnMap = new HashMap<String,
	// AlgorithmParameter>();
	//
	// AlgorithmData algoData = port.getAlgorithmData(algorithmId);
	//
	// ArrayOfAlgorithmParameter algoParamList =
	// port.getAlgorithmParameters(algorithmId);
	//
	// for (AlgorithmParameter algoParam:algoParamList.getAlgorithmParameter())
	// {
	// mapOKBDataTypetoECJDataTypeEnum()
	// port.getDataType(algoParam.getDataTypeId()).getName().getValue()) {
	//
	// }
	// returnMap.put(algoParam.getName().getValue(),
	// port.getDataType(algoParam.getDataTypeId()).getName().getValue());
	// }
	//
	// return returnMap;
	// }

	private <T extends NamedOKBItem> T createNamedOKBItem(Class<T> clazz,
			String name) {
		try {
			T instance = clazz.newInstance();
			instance.setName(factory.createNamedOKBItemName(name));
			return instance;
		} catch (IllegalAccessException ex) {
			throw new IllegalArgumentException(ex);
		} catch (InstantiationException ex) {
			throw new IllegalArgumentException(ex);
		}
	}
	
	public Boolean areAlgoParamsEqual() {
		Boolean retval = false;
		
		
		return retval;
	}

	public void createMissingAlgoParams(Long algorithmId,
			Map<String, Object> ecjAlgoParams) {

		List<String> missingParams = new ArrayList<String>();

		ArrayOfAlgorithmParameter hlAlgoParams = port
				.getAlgorithmParameters(algorithmId);
		Map<String, Object> hlAlgoParamMap = new HashMap<String, Object>();

		for (AlgorithmParameter algoParam : hlAlgoParams
				.getAlgorithmParameter()) {
			hlAlgoParamMap.put(algoParam.getName().getValue(), null); // value
			// hangs
			// on
			// Experiments
		}

		String[] ecjparamNames = (String[]) ecjAlgoParams.keySet().toArray();

		for (int i = 0; i < ecjparamNames.length; i++) {
			if (!hlAlgoParamMap.containsKey(ecjparamNames[i])) {
				missingParams.add(new String(ecjparamNames[i]));
			}
		}

	}
}
